module Analytics
  module Serializer
    class Swift < Base
      # It's named AnalyticsFile instead of Analytics,
      # because it failed to have a class which used the same name as the parent module.
      class AnalyticsFile
        def initialize(src)
          @src = src
        end

        def save(path)
          @file_name = 'Analytics.swift'
          output_path = File.join(path, @file_name)
          File.write(output_path, render)
        end

        private

        def render
          ERB.new(template, nil, '-').result(binding)
        end

        def template
          <<~TEMPLATE
            <%= ERB.new(Analytics::Templates.tmpl_at('header.erb')).result(binding) %>

            import Foundation

            final class Analytics {

                private let loggers: [AnalyticsLogger]

                init(loggers: [AnalyticsLogger]) {
                    self.loggers = loggers
                }
            }

            // MARK: - AnalyticsLogger

            extension Analytics: AnalyticsLogger {

                func set(enabled: Bool) {
                    loggers.forEach { $0.set(enabled: enabled) }
                }

                func log(event: Analytics.Event) {
                    loggers.forEach { $0.log(event: event) }
                }

                func track(screen: Analytics.Screen) {
                    loggers.forEach { $0.track(screen: screen) }
                }

                func set(userProperty: Analytics.UserProperty) {
                    loggers.forEach { $0.set(userProperty: userProperty) }
                }
            }
          TEMPLATE
        end
      end
    end
  end
end
