module Analytics
  module Serializer
    class Swift < Base
      class UserProperty
        def initialize(src)
          @src = src
        end

        def save(path)
          @file_name = 'AnalyticsUserProperty.swift'
          output_path = File.join(path, @file_name)
          File.write(output_path, render)
        end

        private

        def render
          ERB.new(template, nil, '-').result(binding)
        end

        def template
          <<~TEMPLATE
            <%= ERB.new(Analytics::Templates.tmpl_at('header.erb')).result(binding) %>

            import Foundation

            extension Analytics {

                struct UserProperty {

                    let name: String
                    let value: String?

                    private init(name: String, value: String?) {
                        self.name = name
                        self.value = value
                    }
                }
            }

            // MARK: - User properties

            extension Analytics.UserProperty {
            <% @src["userProperties"].each do |user_property| -%>

                /// <%= user_property["description"] %>
                /// - Parameter value: The value of the user property, or `nil` if You want to remove the user property.
                static func <%= user_property["name"].camel_case -%>(with value: <%= user_property["values"].nil? ? 'String?' : user_property["name"].pascal_case %>) -> Analytics.UserProperty {
                    return .init(name: "<%= user_property["name"] -%>", value: <%= user_property["values"].nil? ? 'value' : 'value.rawValue' %>)
                }
            <% end -%>
            }

            extension Analytics.UserProperty {
            <% Interactor::UserProperty.all_enum_user_properties(@src["userProperties"]).each do |user_property| -%>

                enum <%= user_property["name"].pascal_case %>: String {
            <% user_property["values"].each do |value| -%>
                    case <%= value.camel_case %> = "<%= value -%>"
            <% end -%>
                }
            <% end -%>
            }
          TEMPLATE
        end
      end
    end
  end
end
